package atomicwheel;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

/**
 * Stores the named attributes for a certain kind of {@link GraphItem}s.
 * 
 * @param <GI>
 *            {@link GraphItem} for which the attributes are valid
 * 
 * @author matthias
 */
public class AttributeManager<GI extends GraphItem> implements Serializable {
	private static final long serialVersionUID = 1L;

	final Map<String, Attribute<GI, ?>> attributes = new HashMap<String, Attribute<GI, ?>>();

	/**
	 * Creates and registers an attribute. If an attribute with that name
	 * already exists, it is replaced.
	 * 
	 * @param name
	 *            name of the attribute
	 * @param type
	 *            {@link Class} of the attribute's type
	 * @return the new attribute
	 */
	public <T> Attribute<GI, T> createAttribute(final String name,
			final Class<T> type) {
		final Attribute<GI, T> attrib = new Attribute<GI, T>(type);
		add(name, attrib);
		return attrib;
	}

	/**
	 * Obtains the attribute with the given name. Throws a
	 * {@link NoSuchElementException} if it doesn't exists.
	 * 
	 * @param name
	 *            name of the attribute
	 * @return the attribute
	 */
	@SuppressWarnings("unchecked")
	public <T> Attribute<GI, T> get(final String name) {
		if (!attributes.containsKey(name))
			throw new NoSuchElementException(name);
		return (Attribute<GI, T>) attributes.get(name);
	}

	/**
	 * Obtains the attribute with the given name, or creates it if it doesn't
	 * yet exists with the right type. If an attribute with that name but
	 * different type already exists, it is replaced.
	 * 
	 * @param name
	 *            name of the attribute
	 * @param type
	 *            {@link Class} of the attribute's type
	 * @return the attribute
	 */
	@SuppressWarnings("unchecked")
	public <T> Attribute<GI, T> get(final String name, final Class<T> type) {
		if (attributes.containsKey(name) && attributes.get(name).type == type)
			return (Attribute<GI, T>) attributes.get(name);
		return createAttribute(name, type);
	}

	/**
	 * Registers an attribute. If an attribute with that name already exists, it
	 * is replaced.
	 * 
	 * @param name
	 *            name of the attribute
	 * @param attrib
	 *            the attribute itsself
	 */
	public <T> void add(final String name, final Attribute<GI, T> attrib) {
		attributes.put(name, attrib);
	}

	/** @return a map of attribute names to attributes */
	public Map<String, Attribute<GI, ?>> getAttributes() {
		return attributes;
	}
}
