package atomicwheel;

import java.io.Serializable;
import java.util.Collection;

/**
 * An attribute that automatically assigns consecutive, unique IDs to graph
 * items, starting with 0. There is no way to remove items, or change their IDs.
 * 
 * @param <GI>
 *            the {@link GraphItem} to manage
 * 
 * @author matthias
 */
public class IDAttribute<GI extends GraphItem> extends Attribute<GI, Integer>
		implements Serializable {
	private static final long serialVersionUID = 1L;

	private final IDList id = new IDList();

	public IDAttribute() {
		super(Integer.class);
	}

	public IDAttribute(final Collection<GI> items) {
		this();
		add(items);
	}

	/**
	 * Adds an item, assigning it the next available ID.
	 * 
	 * @param item
	 *            the item to add
	 */
	public void add(final GI item) {
		if (values.containsKey(item))
			return;
		values.put(item, id.next());
	}

	/**
	 * Add {@link Collection} of items, assigning IDs in the order that the
	 * iterator returns them.
	 * 
	 * @param items
	 *            the items to add
	 */
	public void add(final Collection<GI> items) {
		for (final GI item : items)
			add(item);
	}

	/** Not supported. */
	@Override
	public void set(final GI item, final Integer value) {
		throw new UnsupportedOperationException();
	}

	private class IDList {
		private int next = 0;

		private synchronized int next() {
			return next++;
		}
	}
}
